@@ -498,6 +498,9 @@ def ncinfo():
 def _nc4tonc3(filename4,filename3,clobber=False,nchunk=10,quiet=False,format='NETCDF3_64BIT'):
     """convert a netcdf 4 file (filename4) in NETCDF4_CLASSIC format
     to a netcdf 3 file (filename3) in NETCDF3_64BIT format."""
+
+    from netCDF4 import Dataset
+
     ncfile4 = Dataset(filename4,'r')
     if ncfile4.file_format != 'NETCDF4_CLASSIC':
         raise IOError('input file must be in NETCDF4_CLASSIC format')
@@ -533,30 +536,30 @@ def _nc4tonc3(filename4,filename3,clobber=False,nchunk=10,quiet=False,format='NE
             FillValue = None
         var = ncfile3.createVariable(varname,ncvar.dtype,ncvar.dimensions,fill_value=FillValue)
         # fill variable attributes.
-    attdict = ncvar.__dict__
-    if '_FillValue' in attdict:
-        del attdict['_FillValue']
-    var.setncatts(attdict)
-    #for attname in ncvar.ncattrs():
-    #    if attname == '_FillValue': continue
-    #    setattr(var,attname,getattr(ncvar,attname))
-    # fill variables with data.
-    if hasunlimdim: # has an unlim dim, loop over unlim dim index.
-        # range to copy
-        if nchunk:
-            start = 0; stop = len(unlimdim); step = nchunk
-            if step < 1:
-                step = 1
-            for n in range(start, stop, step):
-                nmax = n+nchunk
-                if nmax > len(unlimdim):
-                    nmax=len(unlimdim)
-                var[n:nmax] = ncvar[n:nmax]
-        else:
-            var[0:len(unlimdim)] = ncvar[:]
-    else: # no unlim dim or 1-d variable, just copy all data at once.
-        var[:] = ncvar[:]
-    ncfile3.sync() # flush data to disk
+        attdict = ncvar.__dict__
+        if '_FillValue' in attdict:
+            del attdict['_FillValue']
+        var.setncatts(attdict)
+        #for attname in ncvar.ncattrs():
+        #    if attname == '_FillValue': continue
+        #    setattr(var,attname,getattr(ncvar,attname))
+        # fill variables with data.
+        if hasunlimdim: # has an unlim dim, loop over unlim dim index.
+            # range to copy
+            if nchunk:
+                start = 0; stop = len(unlimdim); step = nchunk
+                if step < 1:
+                    step = 1
+                for n in range(start, stop, step):
+                    nmax = n+nchunk
+                    if nmax > len(unlimdim):
+                        nmax=len(unlimdim)
+                    var[n:nmax] = ncvar[n:nmax]
+            else:
+                var[0:len(unlimdim)] = ncvar[:]
+        else: # no unlim dim or 1-d variable, just copy all data at once.
+            var[:] = ncvar[:]
+        ncfile3.sync() # flush data to disk
     # close files.
     ncfile3.close()
     ncfile4.close()
@@ -567,7 +570,7 @@ def nc4tonc3():
 
  usage: %s [-h] [-o] [--chunk] netcdf4filename netcdf3filename
  -h -- Print usage message.
- -o -- Overwite destination file (default is to raise an error if output file already exists).
+ -o -- Overwrite destination file (default is to raise an error if output file already exists).
  --quiet=(0|1)  -- if 1, don't print diagnostic information.
  --format -- netcdf3 format to use (NETCDF3_64BIT by default, can be set to NETCDF3_CLASSIC)
  --chunk=(integer) -- number of records along unlimited dimension to
@@ -772,7 +775,7 @@ def nc3tonc4():
 
  usage: %s [-h] [-o] [--vars=var1,var2,..] [--zlib=(0|1)] [--complevel=(1-9)] [--shuffle=(0|1)] [--fletcher32=(0|1)] [--unpackshort=(0|1)] [--quantize=var1=n1,var2=n2,..] netcdf3filename netcdf4filename
  -h -- Print usage message.
- -o -- Overwite destination file (default is to raise an error if output file already exists).
+ -o -- Overwrite destination file (default is to raise an error if output file already exists).
  --vars -- comma separated list of variable names to copy (default is to copy
     all variables)
  --classic=(0|1) -- use NETCDF4_CLASSIC format instead of NETCDF4 (default 1)
