@@ -44,6 +44,7 @@ class VariablesTestCase(unittest.TestCase):
     def runTest(self):
         """testing multi-file dataset access"""
         f = MFDataset(self.files,check=True)
+        f.set_auto_maskandscale(True) # issue570
         assert f.history == 'created today'
         assert_array_equal(np.arange(0,nx),f.variables['x'][:])
         varin = f.variables['data']
@@ -61,6 +62,12 @@ class VariablesTestCase(unittest.TestCase):
         assert getattr(varin,'nonexistantatt',None) == None
         f.close()
 
+    def test_get_by_mfdataset(self):
+        """testing multi-file get_variables_by_attributes."""
+        f = MFDataset(self.files,check=True)
+        assert f.get_variables_by_attributes(axis='T') == []
+        f.get_variables_by_attributes(units='zlotys')[0] == f['x']
+        f.close()
 
 class NonuniformTimeTestCase(unittest.TestCase):
     ninc = 365
